''''''''''''''''''''''''''''''''''''''''''''''
' Script pour dithering automatique avec ASCOM
'
' La monture est lgrement dplace de manire
' alatoire  intervalles rguliers.
'
' Le script peut se lancer depuis une fenetre de commande avec
' cscript dithering.vbs
'
' Les lignes commenant avec ' sont des commentaires non pris en compte
'
' version: 1.0 - 5/3/2020
' contact: sylvain.chapeland @at@ gmail.com
''''''''''''''''''''''''''''''''''''''''''''''


''''''''''''''''''''''''''''''''''
' Paramtres
''''''''''''''''''''''''''''''''''

' Nom du telescope ASCOM  utiliser
' Laisser vide pour avoir le menu de selection
' ou mettre le nom du driver ascom  utiliser
telescopeName = ""
'telescopeName = "ASCOM.Simulator.Telescope"
'telescopeName = "EQMOD_SIM.Telescope"

' Intervalle de dithering (secondes)
ditheringInterval = 60

' Dlai avant premier dithering (secondes)
ditheringDelay = 0

' Dplacement minimal/maximal RA / DEC (en secondes)
ditheringMinMoveX = 0.0
ditheringMaxMoveX = 1.0

ditheringMinMoveY = 0.0
ditheringMaxMoveY = 1.0

' backlash DEC (en secondes)
' ce temps est ajout au temps de dplacement
' en cas de changement de direction sur l'axe DEC
ditheringDecBacklash = 0

' Fin programme du script
' Temps en secondes, ou zro pour ne jamais sortir
exitTimeout = 0



''''''''''''''''''''''''''''''''''
' Code
''''''''''''''''''''''''''''''''''

' Choose telescope, if not set
If telescopeName = "" Then
  Log "Slectionnez un tlescope ..."
  Set Chooser = CreateObject("ASCOM.Utilities.Chooser")
  telescopeName = Chooser.Choose("")
  If telescopeName = "" Then
    Call TerminateWithError
  End If
End If

' Connect telescope
Log "Connexion  " + telescopeName
Set Telescope = CreateObject(telescopeName)
Telescope.Connected = True
Log "Connexion ok"

' Init
Log "Dmarrage du dithering dans " + CStr(ditheringDelay) + "s, priode = " + CStr(ditheringInterval) + "s"
If exitTimeout > 0 Then
	timeExit = DateAdd("s", exitTimeout, Now())
	Log "Fin programme dans " + CStr(exitTimeout) + " secondes" + " @ " + CStr(timeExit)
End If
timeNextDithering = DateAdd("s", ditheringDelay, Now())
Randomize
TotalX = 0
TotalY = 0
lastX = 0
lastY = 0

' Main loop
Do While True
  t = Now()
  If ExitTimeout > 0 Then
    If t >= timeExit Then
      Exit Do
    End If
  End If  
  If t >= timeNextDithering Then
    timeNextDithering = DateAdd("s", ditheringInterval, timeNextDithering)
  	X = Rnd()* ditheringMaxMoveX + ditheringMinMoveX
  	Y = Rnd()* ditheringMaxMoveY + ditheringMinMoveY
  	If Rnd() < 0.5 Then
  		X = -X
  	End If
  	If Rnd() < 0.5 Then
  		Y = -Y
  	End If
  	TotalX = TotalX + X
  	TotalY = TotalY + Y
  	Log "Dithering RA=" + FormatNumber(X, 2) + " DEC=" + FormatNumber(Y, 2) +"   total RA=" + FormatNumber(TotalX, 2) + " DEC=" +  FormatNumber(TotalY, 2)  	
    lastX = X
  	If (Y > 0) And (lastY < 0) Then
  	  lastY = Y 	
  	  Y = Y + ditheringDecBacklash
  	End If
  	If (Y < 0) And (lastY > 0) Then
  	  lastY = Y
  	  Y = Y - ditheringDecBacklash
  	End If
  	
  	' Move scope
  	If (X>0) Then
  	  	Telescope.PulseGuide 3,Abs(X*1000)
  	Else
  	  	Telescope.PulseGuide 2,Abs(X*1000)
  	End If
  	If (Y>0) Then
  	  	Telescope.PulseGuide 0,Abs(Y*1000)
  	Else
  		Telescope.PulseGuide 1,Abs(Y*1000)
  	End If  	
  End If
  WScript.Sleep 100
Loop

' Program exit
Call Terminate
Sub Terminate
  Log "Fin du script"
  Set Telescope = Nothing
  WScript.Quit
End Sub


' Error handling
Sub TerminateWithError
  Log "Erreur !"
  Call Terminate
End Sub

' Log message with timestamp
Sub Log(Message)
 WScript.Echo CStr(Now()) + "   " + Message
End Sub